@@ -20,11 +20,11 @@
 
 # Read functions #
 
-def AGS4_to_dict(filepath_or_buffer, encoding='utf-8', get_line_numbers=False):
+def AGS4_to_dict(filepath_or_buffer, encoding='utf-8', get_line_numbers=False, rename_duplicate_headers=True):
     """Load all the data in a AGS4 file to a dictionary of dictionaries.
     This GROUP in the AGS4 file is assigned its own dictionary.
 
-    'AGS4_to_dataframe' uses this funtion to load AGS4 data in to Pandas
+    'AGS4_to_dataframe' uses this function to load AGS4 data in to Pandas
     dataframes.
 
     Parameters
@@ -36,6 +36,10 @@ def AGS4_to_dict(filepath_or_buffer, encoding='utf-8', get_line_numbers=False):
     get_line_numbers : bool
         Add line number column to each table (for UNIT, TYPE, and DATA rows) and return
         a dictionary with line numbers for GROUP and HEADING lines (default False)
+    rename_duplicate_headers: bool
+        Rename duplicate headers if found. Neither AGS4 tables nor Pandas
+        dataframes allow duplicate headers, therefore a number will be appended
+        to duplicates to make them unique. (default True)
 
     Returns
     -------
@@ -53,7 +57,7 @@ def AGS4_to_dict(filepath_or_buffer, encoding='utf-8', get_line_numbers=False):
     from rich import print as rprint
     import sys
 
-    if is_file_like(filepath_or_buffer):
+    if _is_file_like(filepath_or_buffer):
         f = filepath_or_buffer
         close_file = False
     else:
@@ -93,32 +97,29 @@ def AGS4_to_dict(filepath_or_buffer, encoding='utf-8', get_line_numbers=False):
                 # Catch HEADER rows with duplicate entries as it will result in a dictionary with
                 # arrays of unequal lengths and cause a ValueError when trying to convert to a
                 # Pandas DataFrame
-                try:
-                    assert len(temp) == len(set(temp))
-                except AssertionError:
-                    rprint(f"[red]  ERROR: HEADER row in [bold]{group}[/bold] (Line {i}) has duplicate entries.[/red]")
+                if len(temp) != len(set(temp)):
 
-                    user_input = input('  Do you want to automatically rename columns and continue (Yes/No) ?')
+                    if rename_duplicate_headers is False:
+                        raise AGS4Error(f"HEADER row in {group} (Line {i}) has duplicate entries")
 
-                    if user_input.lower() not in ['yes', 'y']:
-                        rprint('[red]  File conversion aborted![/red]')
-                        sys.exit()
-                    else:
-                        rprint('[blue]  INFO: Duplicate columns were renamed by appending a number (e.g "_1").[/blue]')
-                        rprint('[yellow]  WARNING: Automatically renamed columns do not conform to AGS4 Rules 19a and 19b.[/yellow]')
-                        rprint('[yellow]           Please review the data and rename or drop duplicate columns as appropriate.[/yellow]')
+                    rprint(f"[yellow]  WARNING: HEADER row in [bold]{group}[/bold] (Line {i}) has duplicate entries.[/yellow]")
 
-                        # Rename duplicates by appending a number
-                        item_count = {}
+                    # Rename duplicate headers by appending a number
+                    item_count = {}
 
-                        for i, item in enumerate(temp):
-                            if item not in item_count:
-                                item_count[item] = {'i': i, 'count': 0}
-                            else:
-                                item_count[item]['i'] = i
-                                item_count[item]['count'] += 1
+                    for i, item in enumerate(temp):
+                        if item not in item_count:
+                            item_count[item] = {'i': i, 'count': 0}
+                        else:
+                            item_count[item]['i'] = i
+                            item_count[item]['count'] += 1
+                            count = item_count[item]['count']
 
-                                temp[i] = temp[i]+'_'+str(item_count[item]['count'])
+                            temp[i] = temp[i]+'_'+str(item_count[item]['count'])
+
+                            rprint(f'[blue]  INFO: Duplicate column {item} found and renamed as {item}_{count}.[/blue]')
+                            rprint('[blue]        Automatically renamed columns do not conform to AGS4 Rules 19a and 19b.[/blue]')
+                            rprint('[blue]        Therefore, please review the data and rename or drop duplicate columns as appropriate.[/blue]')
 
                 # Store HEADING line number
                 line_numbers[group]['HEADING'] = i
@@ -159,7 +160,7 @@ def AGS4_to_dict(filepath_or_buffer, encoding='utf-8', get_line_numbers=False):
     return data, headings
 
 
-def AGS4_to_dataframe(filepath_or_buffer, encoding='utf-8', get_line_numbers=False):
+def AGS4_to_dataframe(filepath_or_buffer, encoding='utf-8', get_line_numbers=False, rename_duplicate_headers=True):
     """Load all the tables in a AGS4 file to a Pandas dataframes. The output is
     a Python dictionary of dataframes with the name of each AGS4 table (i.e.
     GROUP) as the primary key.
@@ -173,6 +174,10 @@ def AGS4_to_dataframe(filepath_or_buffer, encoding='utf-8', get_line_numbers=Fal
     get_line_numbers : bool
         Add line number column to each table (for UNIT, TYPE, and DATA rows) and return
         a dictionary with line numbers for GROUP and HEADING lines (default False)
+    rename_duplicate_headers: bool
+        Rename duplicate headers if found. Neither AGS4 tables nor Pandas
+        dataframes allow duplicate headers, therefore a number will be appended
+        to duplicates to make them unique. (default True)
 
     Returns
     -------
@@ -192,7 +197,8 @@ def AGS4_to_dataframe(filepath_or_buffer, encoding='utf-8', get_line_numbers=Fal
     # Extract AGS4 file into a dictionary of dictionaries
     # A dictionary with group line numbers is returned, in addition to data and headings, for checking purposes
     if get_line_numbers is True:
-        data, headings, line_numbers = AGS4_to_dict(filepath_or_buffer, encoding=encoding, get_line_numbers=get_line_numbers)
+        data, headings, line_numbers = AGS4_to_dict(filepath_or_buffer, encoding=encoding, get_line_numbers=get_line_numbers,
+                                                    rename_duplicate_headers=rename_duplicate_headers)
 
         # Convert dictionary of dictionaries to a dictionary of Pandas dataframes
         df = {}
@@ -202,7 +208,8 @@ def AGS4_to_dataframe(filepath_or_buffer, encoding='utf-8', get_line_numbers=Fal
         return df, headings, line_numbers
 
     # Otherwise only the data and the headings are returned
-    data, headings = AGS4_to_dict(filepath_or_buffer, encoding=encoding)
+    data, headings = AGS4_to_dict(filepath_or_buffer, encoding=encoding,
+                                  rename_duplicate_headers=rename_duplicate_headers)
 
     # Convert dictionary of dictionaries to a dictionary of Pandas dataframes
     df = {}
@@ -212,7 +219,7 @@ def AGS4_to_dataframe(filepath_or_buffer, encoding='utf-8', get_line_numbers=Fal
     return df, headings
 
 
-def AGS4_to_excel(input_file, output_file, encoding='utf-8'):
+def AGS4_to_excel(input_file, output_file, encoding='utf-8', rename_duplicate_headers=True, sort_tables=False):
     """Load all the tables in a AGS4 file to an Excel spreasheet.
 
     Parameters
@@ -221,6 +228,14 @@ def AGS4_to_excel(input_file, output_file, encoding='utf-8'):
         Path to AGS4 file
     output_file : str
         Path to Excel file
+    rename_duplicate_headers: bool
+        Rename duplicate headers if found. Neither AGS4 tables nor Pandas
+        dataframes allow duplicate headers, therefore a number will be appended
+        to duplicates to make them unique. (default False)
+    sort_tables : bool
+        Alphabetically sort worksheets in Excel file. (default False)
+        WARNING: The original order of groups will be lost and cannot be
+        restored when .xlsx file is converted back to .ags.
 
     Returns
     -------
@@ -229,14 +244,28 @@ def AGS4_to_excel(input_file, output_file, encoding='utf-8'):
 
     from pandas import ExcelWriter
     from rich import print as rprint
-    import sys
+    from openpyxl.utils import get_column_letter
 
     # Extract AGS4 file into a dictionary of dictionaries
-    tables, headings = AGS4_to_dataframe(input_file, encoding=encoding)
+    tables, headings = AGS4_to_dataframe(input_file, encoding=encoding,
+                                         rename_duplicate_headers=rename_duplicate_headers)
+
+    # Create list of tables that can be sorted
+    if sort_tables is True:
+        list_of_tables = sorted(tables.keys())
+        rprint('[yellow]WARNING: Worksheets in Excel file will be sorted alphabetically.[/yellow]')
+        rprint('[yellow]         The original group order will not be restored if this .xlsx file is converted back to .ags.[/yellow]')
+    else:
+        list_of_tables = tables.keys()
+
+    # Exit if there is no AGS4 tables in the input file
+    if len(list_of_tables) == 0:
+        rprint(f'[red]  ERROR: No valid AGS4 data found in input file.[/red]')
+        return None
 
     # Write to Excel file
     with ExcelWriter(output_file) as writer:
-        for key in tables:
+        for key in list_of_tables:
             rprint(f'[green]Writing data from... [bold]{key}[/bold][/green]')
 
             # Check table size and issue warning for large files that could crash the program
@@ -246,14 +275,15 @@ def AGS4_to_excel(input_file, output_file, encoding='utf-8'):
                 rprint(f'[yellow]  WARNING: {key} has {tables[key].shape[0]} rows, so it may take a few minutes to export.[/yellow]')
                 rprint('[yellow]           The program will terminate if it runs out of memory in the process.[/yellow]')
 
-                user_input = input('  Do you want to continue (Yes/No) ?')
-
-                if user_input.lower() not in ['yes', 'y']:
-                    rprint('[red]  File conversion aborted![/red]')
-                    sys.exit()
-
             tables[key].to_excel(writer, sheet_name=key, index=False)
 
+            # Update column widths in xlxs file to fit contents
+            for i, col in enumerate(tables[key], start=1):
+                # 13 < colummn_width < 75 characters (approximately)
+                max_width = min(max(13, tables[key][col].map(len).max() + 1), 75)
+
+                writer.sheets[key].column_dimensions[get_column_letter(i)].width = max_width
+
 
 # Write functions #
 
@@ -288,22 +318,28 @@ def dataframe_to_AGS4(data, headings, filepath, mode='w', index=False, encoding=
     # Open file and write/append data
     with open(filepath, mode, newline='', encoding=encoding) as f:
         for key in data:
+            # First make copy of table to avoid unexpected side-effects
+            df = data[key].copy()
+
+            # Take care of an edge case where quoted text is present in a field.
+            # The to_csv function automatically adds an extra pair of quotes
+            # around any quoted strings that is encountered and there is no way
+            # work around it as of Pandas v1.1.5. Therefore, double-double
+            # quotes required by AGS4 Rule 5 are changed to single-double quotes
+            # before the to_csv function is called. This ensures that the output
+            # file has the quoted string in double-double quotes.
+            for col in df.select_dtypes(include='object'):
+                # Loop through columns that contain strings, find entries with '""', and replace
+                # them with '""
+                mask = df[col].str.contains('""', na=False)
+                df.loc[mask, :] = df.loc[mask, :].apply(lambda x: x.str.replace('""', '"'))
 
             try:
                 columns = headings[key]
 
                 rprint(f'[green]Writing data from... [bold]{key}[/bold][green]')
                 f.write('"GROUP"'+","+'"'+key+'"'+'\r\n')
-                data[key].apply(lambda x: x.str.replace('""', '"')).to_csv(f, index=index, quoting=1, columns=columns, line_terminator='\r\n', encoding=encoding)
-                # The lambda funtion is applied to the dataframe to take care of
-                # an edge case where quoted text is present in a field. The
-                # to_csv function automatiically adds an extra pair of quotes
-                # around any quoted strings that is encountered and there is not
-                # way work around for it as of Pandas v1.1.5. Therefore,
-                # double-double quotes required by AGS4 Rule 5 are changed to
-                # single-double quotes before the to_csv function is called.
-                # This ensures that the output file has the quoted string in
-                # double-double quotes.
+                df.to_csv(f, index=index, quoting=1, columns=columns, line_terminator='\r\n', encoding=encoding)
                 f.write("\r\n")
 
             except KeyError:
@@ -316,7 +352,7 @@ def dataframe_to_AGS4(data, headings, filepath, mode='w', index=False, encoding=
                     rprint("[italic yellow]           Please check column order and ensure AGS4 Rule 7 is still satisfied.[/italic yellow]")
 
                 f.write('"GROUP"'+","+'"'+key+'"'+'\r\n')
-                data[key].apply(lambda x: x.str.replace('""', '"')).to_csv(f, index=index, quoting=1, line_terminator='\r\n', encoding=encoding)
+                df.to_csv(f, index=index, quoting=1, line_terminator='\r\n', encoding=encoding)
                 f.write("\r\n")
 
 
@@ -330,7 +366,7 @@ def excel_to_AGS4(input_file, output_file, format_numeric_columns=True, dictiona
         e.g. output from AGS4.AGS4_to_excel)
     output_file : str
         Path to AGS4 file
-    reformat_numeric_columns : bool, optional
+    format_numeric_columns : bool, optional
         Format numeric columns to match specified TYPE
     dictionary : str
         Filepath to dictionary if the UNIT and TYPE data in tables need to be
@@ -347,33 +383,38 @@ def excel_to_AGS4(input_file, output_file, format_numeric_columns=True, dictiona
     # Read data from Excel file in to DataFrames
     tables = read_excel(input_file, sheet_name=None, engine='openpyxl')
 
-    # Format numeric columns
-    if format_numeric_columns is True:
+    # Not all worksheets in the spreadsheet may contain valid AGS4 tables, therefore
+    # initiate variable to keep track of worksheets to export
+    valid_tables = []
 
-        for key in tables:
-            # First drop columns with non-compliant names (AGS Rule 19)
-            columns_to_drop = []
+    for key, df in tables.items():
+        # Assume that only worksheets with a 'HEADING' column contain valid AGS4 data
+        if 'HEADING' in df:
+            valid_tables.append(key)
+        else:
+            rprint(f'[yellow]  WARNING: Worksheet [bold]{key}[/bold] dropped as it does not have a HEADING column.[/yellow]')
+            continue
 
-            for col_name in tables[key].columns:
-                if col_name == 'HEADING':
-                    pass
-                elif col_name.isupper() & (len(col_name) <= 9) & (len(col_name.split('_')[0]) <= 4):
-                    pass
-                else:
-                    columns_to_drop.append(col_name)
-                    rprint(f'[yellow]  WARNING: Column [bold]{col_name}[/bold] dropped as name does not conform to AGS4 Rule 19.[/yellow]')
+        # List column names that don't conform to Rule 19 (using a negative look-ahead regex)
+        for col_name in df.filter(regex=r'^(?!HEADING|^[A-Z0-9]{4}_[A-Z0-9]{1,4}$)', axis='columns'):
+            rprint(f'[yellow]  WARNING: Column [bold]{col_name}[/bold] dropped as name does not conform to AGS4 Rule 19.[/yellow]')
 
-            tables[key].drop(columns=columns_to_drop, inplace=True)
+        # Drop columns that don't conform to Rule 19
+        df = df.filter(regex=r'HEADING|^[A-Z0-9]{4}_[A-Z0-9]{1,4}$', axis='columns')
 
-            # Then drop rows without a proper HEADING
-            tables[key] = tables[key].loc[tables[key].HEADING.isin(['UNIT', 'TYPE', 'DATA']), :]
+        # Drop rows that are not 'UNIT', 'TYPE', or 'DATA'
+        df = df.loc[df.HEADING.isin(['UNIT', 'TYPE', 'DATA']), :]
 
-            # Finally format numeric column
+        # Finally format numeric column if required
+        if format_numeric_columns is True:
             rprint(f'[green]Formatting columns in... [bold]{key}[/bold][/green]')
-            tables[key] = convert_to_text(tables[key], dictionary=dictionary)
+            tables[key] = convert_to_text(df, dictionary=dictionary)
 
     # Export dictionary of DataFrames to AGS4 file
-    dataframe_to_AGS4(tables, {}, output_file, warnings=False)
+    if len(valid_tables) == 0:
+        rprint(f'[red]  ERROR: No valid AGS4 data found in input file. Please see warning messages above.[/red]')
+    else:
+        dataframe_to_AGS4({key: tables[key] for key in valid_tables}, {}, output_file, warnings=False)
 
 
 # Formatting functions #
@@ -460,16 +501,14 @@ def convert_to_text(dataframe, dictionary=None):
     # Check whether to use UNIT and TYPE rows in dataframe or to
     # retrieve values from the dictionary file
     if dictionary is None:
-        try:
-            # Check whether dataframe has "UNIT" and "TYPE" rows
-            assert 'UNIT' in df.HEADING.values
-            assert 'TYPE' in df.HEADING.values
+        # Check whether dataframe has "UNIT" and "TYPE" rows
+        if ('UNIT' in df.HEADING.values) and ('TYPE' in df.HEADING.values):
 
             for col in df.columns:
                 TYPE = df.loc[df.HEADING == 'TYPE', col].values[0]
                 df = format_numeric_column(df, col, TYPE)
 
-        except AssertionError:
+        else:
             rprint("[red]  ERROR: Cannot convert to text as UNIT and/or TYPE row(s) are missing.")
             rprint("[red]         Please provide dictonary file or add UNIT & TYPE rows to input file to proceed.[/red]")
             sys.exit()
@@ -480,18 +519,18 @@ def convert_to_text(dataframe, dictionary=None):
         DICT = temp['DICT']
 
         # Check whether UNIT and TYPE rows are already in dataframe
-        UNIT_row_present = 'UNIT' in df.HEADING.values
-        TYPE_row_present = 'TYPE' in df.HEADING.values
+        is_UNIT_row_present = 'UNIT' in df.HEADING.values
+        is_TYPE_row_present = 'TYPE' in df.HEADING.values
 
         # Format columns and add UNIT/TYPE rows if necessary
         for col in df.columns:
 
             if col == 'HEADING':
 
-                if not UNIT_row_present:
+                if not is_UNIT_row_present:
                     df.loc[-2, 'HEADING'] = 'UNIT'
 
-                if not TYPE_row_present:
+                if not is_TYPE_row_present:
                     df.loc[-1, 'HEADING'] = 'TYPE'
 
             else:
@@ -501,14 +540,14 @@ def convert_to_text(dataframe, dictionary=None):
                     TYPE = DICT.loc[DICT.DICT_HDNG == col, 'DICT_DTYP'].iloc[0]
                     UNIT = DICT.loc[DICT.DICT_HDNG == col, 'DICT_UNIT'].iloc[0]
 
-                    if UNIT_row_present:
+                    if is_UNIT_row_present:
                         # Overwrite existing UNIT with one from the dictionary
                         df.loc[df.HEADING == 'UNIT', col] = UNIT
                     else:
                         # Add UNIT row if one is not already there
                         df.loc[-2, col] = UNIT
 
-                    if TYPE_row_present:
+                    if is_TYPE_row_present:
                         # Overwrite existing TYPE with one from the dictionary
                         df.loc[df.HEADING == 'TYPE', col] = TYPE
                     else:
@@ -559,47 +598,48 @@ def format_numeric_column(dataframe, column_name, TYPE):
             # Apply formatting DATA rows with real numbers. NaNs will be avoided so that they will be exported
             # as "" rather than "nan"
             mask = (df.HEADING == "DATA") & df[col].notna()
-            df.loc[mask, col] = df.loc[mask, col].apply(lambda x: f"{x:.{i}e}")
+            df.loc[mask, col] = df.loc[mask, col].apply(lambda x: f"{x:.{i}E}")
 
         elif 'SF' in TYPE:
 
-            def format_SF(value, TYPE):
-                '''Format a value to specified number of significant figures
-                and return a string.'''
-
-                from numpy import round, log10
-
-                # Avoid log(0) as int(log(0)) will raise an OverflowError
-                if value != 0:
-                    i = int(TYPE.strip('SF')) - 1 - int(log10(abs(value)))
-
-                else:
-                    return f"{value}"
-
-                if i < 0:
-                    return f"{round(value, i):.0f}"
-
-                else:
-                    return f"{value:.{i}f}"
-
             # Apply formatting DATA rows with real numbers. NaNs will be avoided so that they will be exported
             # as "" rather than "nan"
             mask = (df.HEADING == "DATA") & df[col].notna()
-            df.loc[mask, [col]] = df.loc[mask, [col]].applymap(lambda x: format_SF(x, TYPE))
+            df.loc[mask, [col]] = df.loc[mask, [col]].applymap(lambda x: _format_SF(x, TYPE))
 
         else:
             pass
 
     except ValueError:
-        rprint(f"[yellow]  WARNING: Numeric data in [bold]{col}[/bold] exported without reformatting as it had one or more non-numeric entries.[/yellow]")
+        rprint(f"[yellow]  WARNING: Numeric data in [bold]{col:<9}[/bold] not reformatted as it had one or more non-numeric entries.[/yellow]")
 
     except TypeError:
-        rprint(f"[yellow]  WARNING: Numeric data in [bold]{col}[/bold] exported without reformatting as it had one or more non-numeric entries.[/yellow]")
+        rprint(f"[yellow]  WARNING: Numeric data in [bold]{col:<9}[/bold] not reformatted as it had one or more non-numeric entries.[/yellow]")
 
     return df
 
 
-def check_file(input_file, standard_AGS4_dictionary=None):
+def _format_SF(value, TYPE):
+    '''Format a value to specified number of significant figures
+    and return a string.'''
+
+    from math import log10, floor
+
+    # Avoid log(0) as int(log(0)) will raise an OverflowError
+    if value != 0:
+        i = int(TYPE.strip('SF')) - 1 - int(floor(log10(abs(value))))
+
+    else:
+        return f"{value}"
+
+    if i < 0:
+        return f"{round(value, i):.0f}"
+
+    else:
+        return f"{value:.{i}f}"
+
+
+def check_file(input_file, standard_AGS4_dictionary=None, rename_duplicate_headers=True):
     """This function checks the input AGS4 file for errors.
 
     Parameters
@@ -608,6 +648,10 @@ def check_file(input_file, standard_AGS4_dictionary=None):
         Path to AGS4 file (*.ags) to be checked
     standard_AGS4_dict : str
         Path to .ags file with standard AGS4 dictionary
+    rename_duplicate_headers: bool
+        Rename duplicate headers if found. Neither AGS4 tables nor Pandas
+        dataframes allow duplicate headers, therefore a number will be appended
+        to duplicates to make them unique. (default True)
 
     Returns
     -------
@@ -624,6 +668,18 @@ def check_file(input_file, standard_AGS4_dictionary=None):
     # Line checks
     with open(input_file, 'r', newline='', encoding='utf-8', errors='replace') as f:
 
+        # Preflight check for AGS3 files
+        for i, line in enumerate(f, start=1):
+            ags_errors = check.is_ags3_like(line, i, ags_errors=ags_errors)
+
+            # Exit if ags3_like line is found
+            if ('AGS Format Rule 3' in ags_errors) and ('AGS3' in ags_errors['AGS Format Rule 3'][0]['desc']):
+                ags_errors = check.add_meta_data(input_file, standard_AGS4_dictionary, ags_errors=ags_errors)
+                return ags_errors
+
+        # Reset file stream to the beginning to start AGS4 checks
+        f.seek(0)
+
         # Initiate group name and headings list
         group = ''
         headings = []
@@ -664,19 +720,28 @@ def check_file(input_file, standard_AGS4_dictionary=None):
     # Import file into Pandas DataFrame to run group checks
     try:
         rprint('[green]  Loading tables...[/green]')
-        tables, headings, line_numbers = AGS4_to_dataframe(input_file, get_line_numbers=True)
+        tables, headings, line_numbers = AGS4_to_dataframe(input_file, get_line_numbers=True, rename_duplicate_headers=rename_duplicate_headers)
+
+    except AGS4Error as err:
+        rprint('[red] ERROR: Could not continue with group checks on file. Please review error log and fix line errors first.[/red]')
+        raise err
 
     except:
         # TODO: Add specific errors to except clause to conform to flake8
         err = traceback.format_exc()
         rprint('[red] ERROR: Could not continue with group checks on file. Please review error log and fix line errors first.[/red]')
         rprint(f'[red]\n{err}[/red]')
+
+        # Add metadata
+        ags_errors = check.add_meta_data(input_file, standard_AGS4_dictionary, ags_errors=ags_errors)
+
         return ags_errors
 
     # Group Checks
     rprint('[green]  Checking headings and groups...[/green]')
     ags_errors = check.rule_2(tables, headings, line_numbers, ags_errors=ags_errors)
     ags_errors = check.rule_2b(tables, headings, line_numbers, ags_errors=ags_errors)
+    ags_errors = check.rule_8(tables, headings, line_numbers, ags_errors=ags_errors)
     ags_errors = check.rule_12(tables, headings, ags_errors=ags_errors)
     ags_errors = check.rule_13(tables, headings, line_numbers, ags_errors=ags_errors)
     ags_errors = check.rule_14(tables, headings, line_numbers, ags_errors=ags_errors)
@@ -689,8 +754,12 @@ def check_file(input_file, standard_AGS4_dictionary=None):
     if standard_AGS4_dictionary is None:
         standard_AGS4_dictionary = check.pick_standard_dictionary(tables)
 
-    # Combine dictionary file in input file with the standard dictionary to carry out checks
-    dictionary = check.combine_DICT_tables([standard_AGS4_dictionary, input_file])
+    # Import standard dictionary into Pandas DataFrames
+    tables_std_dict, _ = AGS4_to_dataframe(standard_AGS4_dictionary)
+
+    # Combine standard dictionary with DICT table in input file to create an extended dictionary
+    # This extended dictionary is used to check the file schema
+    dictionary = check.combine_DICT_tables(tables_std_dict, tables)
 
     rprint('[green]  Checking file schema...[/green]')
     ags_errors = check.rule_7(headings, dictionary, line_numbers, ags_errors=ags_errors)
@@ -706,16 +775,15 @@ def check_file(input_file, standard_AGS4_dictionary=None):
     ags_errors = check.rule_19b_2(headings, dictionary, line_numbers, ags_errors=ags_errors)
     ags_errors = check.rule_19c(tables, headings, dictionary, line_numbers, ags_errors=ags_errors)
 
-    # Check if file is likely to in AGS3 format
-    ags_errors = check.is_ags3(tables, input_file, ags_errors=ags_errors)
-
     # Add metadata
     ags_errors = check.add_meta_data(input_file, standard_AGS4_dictionary, ags_errors=ags_errors)
 
     return ags_errors
 
 
-def is_file_like(obj):
+# Helper functions/classes #
+
+def _is_file_like(obj):
     """Check if object is file like
 
     Returns
@@ -731,3 +799,7 @@ def is_file_like(obj):
         return False
 
     return True
+
+
+class AGS4Error(Exception):
+    pass
